@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // Basic Usage Environment: for a simple, non-scripted, console application
 // Implementation
 
@@ -32,11 +32,18 @@ BasicTaskScheduler* BasicTaskScheduler::createNew() {
 	return new BasicTaskScheduler();
 }
 
+#define MAX_SCHEDULER_GRANULARITY 10000 // 10 microseconds: We will return to the event loop at least this often
+static void schedulerTickTask(void* clientData) {
+  ((BasicTaskScheduler*)clientData)->scheduleDelayedTask(MAX_SCHEDULER_GRANULARITY, schedulerTickTask, clientData);
+}
+
 BasicTaskScheduler::BasicTaskScheduler()
   : fMaxNumSockets(0) {
   FD_ZERO(&fReadSet);
   FD_ZERO(&fWriteSet);
   FD_ZERO(&fExceptionSet);
+
+  schedulerTickTask(this); // ensures that we handle events frequently
 }
 
 BasicTaskScheduler::~BasicTaskScheduler() {
@@ -145,8 +152,40 @@ void BasicTaskScheduler::SingleStep(unsigned maxDelayTime) {
     if (handler == NULL) fLastHandledSocketNum = -1;//because we didn't call a handler
   }
 
-  // Also handle any delayed event that may have come due.  (Note that we do this *after* calling a socket
-  // handler, in case the delayed event handler modifies the set of readable sockets.)
+  // Also handle any newly-triggered event (Note that we do this *after* calling a socket handler,
+  // in case the triggered event handler modifies The set of readable sockets.)
+  if (fTriggersAwaitingHandling != 0) {
+    if (fTriggersAwaitingHandling == fLastUsedTriggerMask) {
+      // Common-case optimization for a single event trigger:
+      fTriggersAwaitingHandling = 0;
+      if (fTriggeredEventHandlers[fLastUsedTriggerNum] != NULL) {
+	(*fTriggeredEventHandlers[fLastUsedTriggerNum])(fTriggeredEventClientDatas[fLastUsedTriggerNum]);
+      }
+    } else {
+      // Look for an event trigger that needs handling (making sure that we make forward progress through all possible triggers):
+      unsigned i = fLastUsedTriggerNum;
+      EventTriggerId mask = fLastUsedTriggerMask;
+
+      do {
+	i = (i+1)%MAX_NUM_EVENT_TRIGGERS;
+	mask >>= 1;
+	if (mask == 0) mask = 0x80000000;
+
+	if ((fTriggersAwaitingHandling&mask) != 0) {
+	  fTriggersAwaitingHandling &=~ mask;
+	  if (fTriggeredEventHandlers[i] != NULL) {
+	    (*fTriggeredEventHandlers[i])(fTriggeredEventClientDatas[i]);
+	  }
+
+	  fLastUsedTriggerMask = mask;
+	  fLastUsedTriggerNum = i;
+	  break;
+	}
+      } while (i != fLastUsedTriggerNum);
+    }
+  }
+
+  // Also handle any delayed event that may have come due.
   fDelayQueue.handleAlarm();
 }
 
