@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // A RTSP server
 // Implementation
 
@@ -474,30 +474,40 @@ void RTSPServer::RTSPClientSession::handleRequestBytes(int newBytesRead) {
 #endif
     // The request was not (valid) RTSP, but check for a special case: HTTP commands (for setting up RTSP-over-HTTP tunneling):
     char sessionCookie[RTSP_PARAM_STRING_MAX];
-    if (parseHTTPRequestString(cmdName, sizeof cmdName, sessionCookie, sizeof sessionCookie)) {
+    char acceptStr[RTSP_PARAM_STRING_MAX];
+    if (parseHTTPRequestString(cmdName, sizeof cmdName,
+			       urlSuffix, sizeof urlPreSuffix,
+			       sessionCookie, sizeof sessionCookie,
+			       acceptStr, sizeof acceptStr)) {
 #ifdef DEBUG
-      fprintf(stderr, "parseHTTPRequestString() succeeded, returning cmdName \"%s\", sessionCookie \"%s\"\n", cmdName, sessionCookie);
+      fprintf(stderr, "parseHTTPRequestString() succeeded, returning cmdName \"%s\", urlSuffix \"%s\", sessionCookie \"%s\", acceptStr \"%s\"\n", cmdName, urlSuffix, sessionCookie, acceptStr);
 #endif
       // Check that the HTTP command is valid for RTSP-over-HTTP tunneling: There must be a 'session cookie'.
-      Boolean isValidHTTPTunnelingCmd = True;
+      Boolean isValidHTTPCmd = True;
       if (sessionCookie[0] == '\0') {
-	isValidHTTPTunnelingCmd = False;
+	// There was no "x-sessionCookie:" header.  If there was an "Accept: application/x-rtsp-tunnelled" header,
+	// then this is a bad tunneling request.  Otherwise, assume that it's an attempt to access the stream via HTTP.
+	if (strcmp(acceptStr, "application/x-rtsp-tunnelled") == 0) {
+	  isValidHTTPCmd = False;
+	} else {
+	  handleHTTPCmd_StreamingGET(urlSuffix, (char const*)fRequestBuffer);
+	}
       } else if (strcmp(cmdName, "GET") == 0) {
-	handleHTTPCmd_GET(sessionCookie);
+	handleHTTPCmd_TunnelingGET(sessionCookie);
       } else if (strcmp(cmdName, "POST") == 0) {
 	// We might have received additional data following the HTTP "POST" command - i.e., the first Base64-encoded RTSP command.
 	// Check for this, and handle it if it exists:
 	unsigned char const* extraData = fLastCRLF+4;
 	unsigned extraDataSize = &fRequestBuffer[fRequestBytesAlreadySeen] - extraData;
-	if (handleHTTPCmd_POST(sessionCookie, extraData, extraDataSize)) {
+	if (handleHTTPCmd_TunnelingPOST(sessionCookie, extraData, extraDataSize)) {
 	  // We don't respond to the "POST" command, and we go away:
 	  delete this;
 	  return;
 	}
       } else {
-	isValidHTTPTunnelingCmd = False;
+	isValidHTTPCmd = False;
       }
-      if (!isValidHTTPTunnelingCmd) {
+      if (!isValidHTTPCmd) {
 	handleHTTPCmd_notSupported();
       }
     } else {
@@ -955,19 +965,27 @@ void RTSPServer::RTSPClientSession
 			  char const* urlPreSuffix, char const* urlSuffix,
 			  char const* cseq, char const* fullRequestStr) {
   // This will either be:
+  // - an operation on the entire server, if "urlPreSuffix" is "", and "urlSuffix" is "*" (i.e., the special "*" URL), or
   // - a non-aggregated operation, if "urlPreSuffix" is the session (stream)
   //   name and "urlSuffix" is the subsession (track) name, or
-  // - a aggregated operation, if "urlSuffix" is the session (stream) name,
-  //   or "urlPreSuffix" is the session (stream) name, and "urlSuffix"
-  //   is empty.
-  // First, figure out which of these it is:
-  if (fOurServerMediaSession == NULL) { // There wasn't a previous SETUP!
+  // - an aggregated operation, if "urlSuffix" is the session (stream) name,
+  //   or "urlPreSuffix" is the session (stream) name, and "urlSuffix" is empty.
+  // Begin by figuring out which of these it is:
+  ServerMediaSubsession* subsession;
+  if (urlPreSuffix[0] == '\0' && urlSuffix[0] == '*' && urlSuffix[1] == '\0') {
+    // An operation on the entire server.  This works only for GET_PARAMETER and SET_PARAMETER:
+    if (strcmp(cmdName, "GET_PARAMETER") == 0) {
+      handleCmd_GET_PARAMETER(NULL, cseq, fullRequestStr);
+    } else if (strcmp(cmdName, "SET_PARAMETER") == 0) {
+      handleCmd_SET_PARAMETER(NULL, cseq, fullRequestStr);
+    } else {
+      handleCmd_notSupported(cseq);
+    }
+    return;
+  } else if (fOurServerMediaSession == NULL) { // There wasn't a previous SETUP!
     handleCmd_notSupported(cseq);
     return;
-  }
-  ServerMediaSubsession* subsession;
-  if (urlSuffix[0] != '\0' &&
-      strcmp(fOurServerMediaSession->streamName(), urlPreSuffix) == 0) {
+  } else if (urlSuffix[0] != '\0' && strcmp(fOurServerMediaSession->streamName(), urlPreSuffix) == 0) {
     // Non-aggregated operation.
     // Look up the media subsession whose track id is "urlSuffix":
     ServerMediaSubsessionIterator iter(*fOurServerMediaSession);
@@ -1072,11 +1090,18 @@ void RTSPServer::RTSPClientSession
     duration = -duration;
   }
 
-  if (rangeEnd <= 0.0 || rangeEnd > duration) rangeEnd = duration;
-  if (rangeStart < 0.0) {
-    rangeStart = 0.0;
-  } else if (rangeEnd > 0.0 && scale > 0.0 && rangeStart > rangeEnd) {
+  // Make sure that "rangeStart" and "rangeEnd" (from the client's "Range:" header) have sane values
+  // before we send back our own "Range:" header in our response:
+  if (rangeStart < 0.0) rangeStart = 0.0;
+  else if (rangeStart > duration) rangeStart = duration;
+  if (rangeEnd < 0.0) rangeEnd = 0.0;
+  else if (rangeEnd > duration) rangeEnd = duration;
+  if ((scale > 0.0 && rangeStart > rangeEnd && rangeEnd > 0.0) ||
+      (scale < 0.0 && rangeStart < rangeEnd)) {
+    // "rangeStart" and "rangeEnd" were the wrong way around; swap them:
+    double tmp = rangeStart;
     rangeStart = rangeEnd;
+    rangeEnd = tmp;
   }
 
   char* rangeHeader;
@@ -1111,9 +1136,15 @@ void RTSPServer::RTSPClientSession
 						    scale);
       }
       if (sawRangeHeader) {
+	double streamDuration = 0.0; // by default; means: stream until the end of the media
+	if (rangeEnd > 0.0 && (rangeEnd+0.001) < duration) { // the 0.001 is because we limited the values to 3 decimal places
+	  // We want the stream to end early.  Set the duration we want:
+	  streamDuration = rangeEnd - rangeStart;
+	  if (streamDuration < 0.0) streamDuration = -streamDuration; // should happen only if scale < 0.0
+	}
 	fStreamStates[i].subsession->seekStream(fOurSessionId,
 						fStreamStates[i].streamToken,
-						rangeStart);
+						rangeStart, streamDuration);
       }
     }
   }
@@ -1192,10 +1223,11 @@ void RTSPServer::RTSPClientSession
 }
 
 void RTSPServer::RTSPClientSession
-::handleCmd_GET_PARAMETER(ServerMediaSubsession* subsession, char const* cseq,
+::handleCmd_GET_PARAMETER(ServerMediaSubsession* /*subsession*/, char const* cseq,
 			  char const* /*fullRequestStr*/) {
-  // We implement "GET_PARAMETER" just as a 'keep alive',
-  // and send back an empty response:
+  // By default, we implement "GET_PARAMETER" just as a 'keep alive', and send back an empty response.
+  // (If you want to handle "GET_PARAMETER" properly, you can do so by defining a subclass of "RTSPServer"
+  // and "RTSPServer::RTSPClientSession", and then reimplement this virtual function in your subclass.)
   snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
 	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %08X\r\n\r\n",
 	   cseq, dateHeader(), fOurSessionId);
@@ -1204,8 +1236,12 @@ void RTSPServer::RTSPClientSession
 void RTSPServer::RTSPClientSession
 ::handleCmd_SET_PARAMETER(ServerMediaSubsession* /*subsession*/, char const* cseq,
 			  char const* /*fullRequestStr*/) {
-  // By default, we don't implement "SET_PARAMETER":
-  handleCmd_notSupported(cseq);
+  // By default, we implement "SET_PARAMETER" just as a 'keep alive', and send back an empty response.
+  // (If you want to handle "SET_PARAMETER" properly, you can do so by defining a subclass of "RTSPServer"
+  // and "RTSPServer::RTSPClientSession", and then reimplement this virtual function in your subclass.)
+  snprintf((char*)fResponseBuffer, sizeof fResponseBuffer,
+	   "RTSP/1.0 200 OK\r\nCSeq: %s\r\n%sSession: %08X\r\n\r\n",
+	   cseq, dateHeader(), fOurSessionId);
 }
 
 static void lookForHeader(char const* headerName, char const* source, unsigned sourceLen, char* resultStr, unsigned resultMaxSize) {
@@ -1231,7 +1267,9 @@ static void lookForHeader(char const* headerName, char const* source, unsigned s
 }
 
 Boolean RTSPServer::RTSPClientSession::parseHTTPRequestString(char* resultCmdName, unsigned resultCmdNameMaxSize,
-							      char* sessionCookie, unsigned sessionCookieMaxSize) {
+							      char* urlSuffix, unsigned urlSuffixMaxSize,
+							      char* sessionCookie, unsigned sessionCookieMaxSize,
+							      char* acceptStr, unsigned acceptStrMaxSize) {
   // Check for the limited HTTP requests that we expect for specifying RTSP-over-HTTP tunneling.
   // This parser is currently rather dumb; it should be made smarter #####
   char const* reqStr = (char const*)fRequestBuffer;
@@ -1263,8 +1301,20 @@ Boolean RTSPServer::RTSPClientSession::parseHTTPRequestString(char* resultCmdNam
   }
   if (!parseSucceeded) return False;
 
+  // Get the 'URL suffix' that occurred before this:
+  unsigned k = i-6;
+  while (k > 0 && reqStr[k] == ' ') --k; // back up over white space
+  unsigned j = k;
+  while (j > 0 && reqStr[j] != ' ' && reqStr[j] != '/') --j;
+  // The URL suffix is in position (j,k]:
+  if (k - j + 1 > urlSuffixMaxSize) return False; // there's no room> 
+  unsigned n = 0;
+  while (++j <= k) urlSuffix[n++] = reqStr[j];
+  urlSuffix[n] = '\0';
+
   // Look for various headers that we're interested in:
   lookForHeader("x-sessioncookie", &reqStr[i], reqStrSize-i, sessionCookie, sessionCookieMaxSize);
+  lookForHeader("Accept", &reqStr[i], reqStrSize-i, acceptStr, acceptStrMaxSize);
 
   return True;
 }
@@ -1275,7 +1325,7 @@ void RTSPServer::RTSPClientSession::handleHTTPCmd_notSupported() {
 	   dateHeader());
 }
 
-void RTSPServer::RTSPClientSession::handleHTTPCmd_GET(char const* sessionCookie) {
+void RTSPServer::RTSPClientSession::handleHTTPCmd_TunnelingGET(char const* sessionCookie) {
   // Record ourself as having this 'session cookie', so that a subsequent HTTP "POST" command (with the same 'session cookie')
   // can find us:
   if (fOurServer.fClientSessionsForHTTPTunneling == NULL) {
@@ -1297,7 +1347,7 @@ void RTSPServer::RTSPClientSession::handleHTTPCmd_GET(char const* sessionCookie)
 }
 
 Boolean RTSPServer::RTSPClientSession
-::handleHTTPCmd_POST(char const* sessionCookie, unsigned char const* extraData, unsigned extraDataSize) {
+::handleHTTPCmd_TunnelingPOST(char const* sessionCookie, unsigned char const* extraData, unsigned extraDataSize) {
   // Use the "sessionCookie" string to look up the separate "RTSPClientSession" object that should have been used to handle
   // an earlier HTTP "GET" request:
   RTSPServer::RTSPClientSession* prevClientSession
@@ -1318,6 +1368,11 @@ Boolean RTSPServer::RTSPClientSession
   return True;
 }
 
+void RTSPServer::RTSPClientSession::handleHTTPCmd_StreamingGET(char const* /*urlSuffix*/, char const* /*fullRequestStr*/) {
+  // By default, we don't support requests to access streams via HTTP:
+  handleHTTPCmd_notSupported();
+}
+
 static Boolean parseAuthorizationHeader(char const* buf,
 					char const*& username,
 					char const*& realm,
@@ -1440,6 +1495,9 @@ Boolean RTSPServer::RTSPClientSession
 }
 
 void RTSPServer::RTSPClientSession::noteLiveness() {
+#ifdef DEBUG
+  fprintf(stderr, "Liveness indication from client at %s\n", our_inet_ntoa(fClientAddr.sin_addr));
+#endif
   if (fOurServer.fReclamationTestSeconds > 0) {
     envir().taskScheduler()
       .rescheduleDelayedTask(fLivenessCheckTask,
